/* vim: set sw=4 sts=4 et foldmethod=syntax : */

/*
 * Copyright (c) 2006 Ciaran McCreesh <ciaranm@ciaranm.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the library nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <libwrapiter/libwrapiter.hh>
#include <test/test_framework.hh>
#include <test/test_runner.hh>
#include <set>
#include <vector>
#include <map>
#include <algorithm>
#include <numeric>

using namespace test;
using namespace libwrapiter;

/**
 * \file
 * Test cases for libwrapiter_output_iterator.hh .
 */

namespace test_cases
{
    /**
     * Test class.
     */
    class MyClass
    {
        private:
            MyClass(const MyClass &);
            const MyClass & operator= (const MyClass &);

            struct Impl;
            Impl * _impl;

        public:
            MyClass();
            ~MyClass();

            typedef libwrapiter::OutputIterator<MyClass, int> Inserter;
            Inserter inserter() const;

            int sum() const;
    };

    /**
     * Test pair class.
     */
    class MyPairClass
    {
        private:
            MyPairClass(const MyPairClass &);
            const MyPairClass & operator= (const MyPairClass &);

            struct Impl;
            Impl * _impl;

        public:
            MyPairClass();
            ~MyPairClass();

            typedef libwrapiter::OutputIterator<MyClass, std::pair<std::string, std::string> > Inserter;
            Inserter inserter() const;
    };

    /**
     * \test libwrapiter tests.
     */
    struct LibWrapIterInserterTest : TestCase
    {
        LibWrapIterInserterTest() : TestCase("libwrapiter inserter test") { }

        void run()
        {
            MyClass c;
            MyClass::Inserter i(c.inserter());
            *i++ = 1;
            *i++ = 2;
            *i++ = 3;
            TEST_CHECK_EQUAL(c.sum(), 6);
        }
    } test_case_libwrapiter_inserter;

    /**
     * \test libwrapiter tests.
     */
    struct LibWrapIterPairInserterTest : TestCase
    {
        LibWrapIterPairInserterTest() : TestCase("libwrapiter pair inserter test") { }

        void run()
        {
            MyPairClass c;
            MyPairClass::Inserter i(c.inserter());
            *i++ = std::make_pair("foo", "foofoo");
            *i++ = std::make_pair(std::string("bar"), std::string("barbar"));
        }
    } test_case_libwrapiter_pair_inserter;

    /**
     * Implementation details for MyClass.
     */
    struct MyClass::Impl
    {
        std::vector<int> v;
    };

    MyClass::MyClass() :
        _impl(new Impl)
    {
    }

    MyClass::~MyClass()
    {
        delete _impl;
    }

    MyClass::Inserter
    MyClass::inserter() const
    {
        return Inserter(std::back_inserter(_impl->v));
    }

    int
    MyClass::sum() const
    {
        return std::accumulate(_impl->v.begin(), _impl->v.end(), 0);
    }

    /**
     * Implementation details for MyPairClass.
     */
    struct MyPairClass::Impl
    {
        std::map<std::string, std::string> m;
    };

    MyPairClass::MyPairClass() :
        _impl(new Impl)
    {
    }

    MyPairClass::~MyPairClass()
    {
        delete _impl;
    }

    MyPairClass::Inserter
    MyPairClass::inserter() const
    {
        return Inserter(std::inserter(_impl->m, _impl->m.begin()));
    }
}



